package App::SFDC::Metadata;
# ABSTRACT: Metadata tools module for App::SFDC
use strict;
use warnings;
our $VERSION = '0.20'; # VERSION
1;

__END__

=pod

=head1 NAME

App::SFDC::Metadata - Metadata tools module for App::SFDC

=head1 VERSION

version 0.20

=head1 DESCRIPTION

App::SFDC::Metadata provides two commands to App::SFDC: Retrieve and Deploy.
For thorough information on how they work, visit App::SFDC::Command::Deploy
and App::SFDC::Command::Retrieve.

=head1 AUTHOR

Alexander Brett <alexander.brett@sophos.com> L<http://alexander-brett.co.uk>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2015 by Sophos Limited L<https://www.sophos.com/>.

This is free software, licensed under:

  The MIT (X11) License

The full text of the license can be found in the
F<LICENSE> file included with this distribution.

=cut
